@@ -6,7 +6,7 @@ import textwrap
 from operator import attrgetter
 from math import log10
 
-from typing import Optional, Union
+from typing import Optional, Union, List
 
 from AnyQt.QtWidgets import (
     QWidget, QDialog, QVBoxLayout, QSizePolicy, QApplication, QStyle,
@@ -15,8 +15,8 @@ from AnyQt.QtWidgets import (
     QHBoxLayout)
 from AnyQt.QtCore import (
     Qt, QObject, QEvent, QRect, QMargins, QByteArray, QDataStream, QBuffer,
-    QSettings, QUrl, QThread, pyqtSignal as Signal, QSize)
-from AnyQt.QtGui import QIcon, QKeySequence, QDesktopServices, QPainter, QColor
+    QSettings, QUrl, QThread, pyqtSignal as Signal, QSize, QLine)
+from AnyQt.QtGui import QIcon, QKeySequence, QDesktopServices, QPainter, QColor, QPen
 
 from orangewidget import settings, gui
 from orangewidget.report import Report
@@ -47,6 +47,36 @@ __all__ = [
 ]
 
 
+class Message:
+    """
+    A user message.
+
+    :param str text: Message text
+    :param str persistent_id:
+        A persistent message id.
+    :param icon: Message icon
+    :type icon: QIcon or QStyle.StandardPixmap
+    :param str moreurl:
+        An url to open when a user clicks a 'Learn more' button.
+
+    .. seealso:: :const:`OWBaseWidget.UserAdviceMessages`
+    """
+    #: QStyle.SP_MessageBox* pixmap enums repeated for easier access
+    Question = QStyle.SP_MessageBoxQuestion
+    Information = QStyle.SP_MessageBoxInformation
+    Warning = QStyle.SP_MessageBoxWarning
+    Critical = QStyle.SP_MessageBoxCritical
+
+    def __init__(self, text, persistent_id, icon=None, moreurl=None):
+        assert isinstance(text, str)
+        assert isinstance(icon, (type(None), QIcon, QStyle.StandardPixmap))
+        assert persistent_id is not None
+        self.text = text
+        self.icon = icon
+        self.moreurl = moreurl
+        self.persistent_id = persistent_id
+
+
 def _asmappingproxy(mapping):
     if isinstance(mapping, types.MappingProxyType):
         return mapping
@@ -87,81 +117,157 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
                    WidgetMessagesMixin, WidgetSignalsMixin,
                    metaclass=WidgetMetaClass, openclass=True):
     """
-    Base widget class in an orange widget workflow.
+    Base widget class in an Orange widget workflow.
     """
 
-    # Global widget count
-    widget_id = 0
+    #: Widget name, as presented in the Canvas.
+    name: str = None
 
-    # Widget Meta Description
-    # -----------------------
+    #: Short widget description, displayed in canvas help tooltips.
+    description: str = ""
 
-    #: Widget name (:class:`str`) as presented in the Canvas
-    name = None
-    short_name = None
-    id = None
-    category = None
-    version = None
-    #: Short widget description (:class:`str` optional), displayed in
-    #: canvas help tooltips.
-    description = ""
-    #: Widget icon path relative to the defining module
-    icon = "icons/Unknown.png"
-    #: Widget priority used for sorting within a category
-    #: (default ``sys.maxsize``).
-    priority = sys.maxsize
+    #: Widget icon path, relative to the defining module.
+    icon: str = "icons/Unknown.png"
 
-    help = None
-    help_ref = None
-    url = None
-    keywords = []
-    background = None
-    replaces = None
+    class Inputs:
+        """
+        Define inputs in this nested class as class variables.
+        (type `orangewidget.widget.Input`)
 
-    #: A list of published input definitions
-    inputs = []
-    #: A list of published output definitions
-    outputs = []
+        Example::
 
-    # Default widget GUI layout settings
-    # ----------------------------------
+            class Inputs:
+                data = Input("Data", Table)
 
-    #: Should the widget have basic layout
-    #: (If this flag is false then the `want_main_area` and
-    #: `want_control_area` are ignored).
+        Then, register input handler methods with decorators.
+
+        Example::
+
+            @Inputs.data
+            def set_data(self, data):
+                self.my_data = data
+        """
+
+    class Outputs:
+        """
+        Define outputs in this nested class as class variables.
+        (type `orangewidget.widget.Output`)
+
+        Example::
+
+            class Outputs:
+                data = Output("Data", Table)
+
+        Then, send results to the output with its `send` method.
+
+        Example::
+
+            def commit(self):
+                self.Outputs.data.send(self.my_data)
+        """
+
+    # -------------------------------------------------------------------------
+    # Widget GUI Layout Settings
+    # -------------------------------------------------------------------------
+
+    #: Should the widget have basic layout?
+    #: (if not, the rest of the GUI layout settings are ignored)
     want_basic_layout = True
-    #: Should the widget construct a `mainArea` (this is a resizable
-    #: area to the right of the `controlArea`).
-    want_main_area = True
-    #: Should the widget construct a `controlArea`.
+
+    #: Should the widget construct a `controlArea`?
     want_control_area = True
-    #: Is vertical scrolling on the widget's left side, which usually
-    #: contains  the `controlArea`, allowed?
-    left_side_scrolling = False
+
+    #: Should the widget construct a `mainArea`?
+    #: (a resizable area to the right of the `controlArea`)
+    want_main_area = True
+
+    #: Should the widget construct a `message_bar`?
+    #: (if not, make sure you show warnings/errors in some other way)
+    want_message_bar = True
+
+    #: Should the widget's window be resizeable?
+    #: (if not, the widget will derive a fixed size constraint from its layout)
+    resizing_enabled = True
+
+    #: Should the widget remember its window position/size?
+    save_position = True
+
     #: Orientation of the buttonsArea box; valid only if
     #: `want_control_area` is `True`. Possible values are Qt.Horizontal,
     #: Qt.Vertical and None for no buttons area
     buttons_area_orientation = Qt.Horizontal
-    #: Specify whether the default message bar widget should be created
-    #: and placed into the default layout. If False then clients are
-    #: responsible for displaying messages within the widget in an
-    #: appropriate manner.
-    want_message_bar = True
+
+    #: A list of advice messages to display to the user.
+    #: (when a widget is first shown a message from this list is selected
+    #: for display. If a user accepts (clicks 'Ok. Got it') the choice is
+    #: recorded and the message is never shown again (closing the message
+    #: will not mark it as seen). Messages can be displayed again by pressing
+    #: Shift + F1)
+    UserAdviceMessages: List[Message] = []
+
+    # -------------------------------------------------------------------------
+    # Miscellaneous Options
+    # -------------------------------------------------------------------------
+
+    #: Version of the settings representation
+    #: (subclasses should increase this number when they make breaking
+    #: changes to settings representation (a settings that used to store
+    #: int now stores string) and handle migrations in migrate and
+    #: migrate_context settings)
+    settings_version: int = 1
+
+    #: Signal emitted before settings are packed and saved.
+    #: (gives you a chance to sync state to Setting values)
+    settingsAboutToBePacked = Signal()
+
+    #: Settings handler, can be overridden for context handling.
+    settingsHandler: SettingsHandler = None
+
+    #: Widget keywords, used for finding it in the quick menu.
+    keywords: List[str] = []
+
+    #: Widget priority, used for sorting within a category.
+    priority: int = sys.maxsize
+
+    #: Short name for widget, displayed in toolbox.
+    #: (set this if the widget's conventional name is long)
+    short_name: str = None
+
+    #: A list of widget IDs that this widget replaces in older workflows.
+    replaces: List[str] = None
+
     #: Widget painted by `Save graph` button
-    graph_name = None
-    graph_writers = [f for f in ImgFormat.formats
-                     if getattr(f, 'write_image', None)
-                     and getattr(f, "EXTENSIONS", None)]
+    graph_name: str = None
+    graph_writers: List[ImgFormat] = [f for f in ImgFormat.formats
+                                  if getattr(f, 'write_image', None)
+                                  and getattr(f, "EXTENSIONS", None)]
 
-    save_position = True
+    #: Explicitly set widget category,
+    #: should it not already be part of a package.
+    category: str = None
 
-    #: If false the widget will receive fixed size constraint
-    #: (derived from it's layout). Use for widgets which have simple
-    #: static size contents.
-    resizing_enabled = True
+    # -------------------------------------------------------------------------
+    # Private Interface
+    # -------------------------------------------------------------------------
 
-    blockingStateChanged = Signal(bool)
-    processingStateChanged = Signal(int)
+    # Global widget count
+    widget_id = 0
+
+    # Custom widget id, kept for backward compatibility
+    id = None
+
+    # A list of published input definitions.
+    # (conventionally generated from Inputs nested class)
+    inputs = []
+
+    # A list of published output definitions.
+    # (conventionally generated from Outputs nested class)
+    outputs = []
+
+    contextAboutToBeOpened = Signal(object)
+    contextOpened = Signal()
+    contextClosed = Signal()
+    openVisualSettingsClicked = Signal()
 
     # Signals have to be class attributes and cannot be inherited,
     # say from a mixin. This has something to do with the way PyQt binds them
@@ -169,35 +275,9 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
     messageActivated = Signal(Msg)
     messageDeactivated = Signal(Msg)
 
-    settingsHandler = None
-    """:type: SettingsHandler"""
-
-    #: Version of the settings representation
-    #: Subclasses should increase this number when they make breaking
-    #: changes to settings representation (a settings that used to store
-    #: int now stores string) and handle migrations in migrate and
-    #: migrate_context settings.
-    settings_version = 1
-
     savedWidgetGeometry = settings.Setting(None)
     controlAreaVisible = settings.Setting(True, schema_only=True)
 
-    #: A list of advice messages (:class:`Message`) to display to the user.
-    #: When a widget is first shown a message from this list is selected
-    #: for display. If a user accepts (clicks 'Ok. Got it') the choice is
-    #: recorded and the message is never shown again (closing the message
-    #: will not mark it as seen). Messages can be displayed again by pressing
-    #: Shift + F1
-    #:
-    #: :type: list of :class:`Message`
-    UserAdviceMessages = []
-
-    settingsAboutToBePacked = Signal()
-    contextAboutToBeOpened = Signal(object)
-    contextOpened = Signal()
-    contextClosed = Signal()
-    openVisualSettingsClicked = Signal()
-
     # pylint: disable=protected-access, access-member-before-definition
     def __new__(cls, *args, captionTitle=None, **kwargs):
         self = super().__new__(cls, None, cls.get_flags())
@@ -206,6 +286,13 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
         WidgetMessagesMixin.__init__(self)
         WidgetSignalsMixin.__init__(self)
 
+        # handle deprecated left_side_scrolling
+        if hasattr(self, 'left_side_scrolling'):
+            warnings.warn(
+                "'OWBaseWidget.left_side_scrolling' is deprecated.",
+                DeprecationWarning
+            )
+
         stored_settings = kwargs.get('stored_settings', None)
         if self.settingsHandler:
             self.settingsHandler.initialize(self, stored_settings)
@@ -293,8 +380,7 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
             return None
         properties = {name: getattr(cls, name) for name in
                       ("name", "icon", "description", "priority", "keywords",
-                       "help", "help_ref", "url",
-                       "version", "background", "replaces", "short_name")}
+                       "replaces", "short_name", "category")}
         properties["id"] = cls.id or cls.__module__
         properties["inputs"] = cls.get_signals("inputs")
         properties["outputs"] = cls.get_signals("outputs")
@@ -311,7 +397,7 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
 
         def __init__(self, *args, **kwargs):
             super().__init__(*args, *kwargs)
-            self.setHandleWidth(8)
+            self.setHandleWidth(18)
 
         def _adjusted_size(self, size_method):
             size = size_method(super())()
@@ -357,17 +443,23 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
             def paintEvent(self, event):
                 super(QSplitterHandle, self).paintEvent(event)
                 painter = QPainter(self)
-                painter.setPen(QColor(160, 160, 160))
-                w = self.width() - 2
+                pen = QPen()
+                pen.setColor(QColor(160, 160, 160))
+                pen.setCapStyle(Qt.RoundCap)
+                pen.setWidth(3)
+                painter.setPen(pen)
+                w = self.width() - 6
                 if self.controlAreaShown:
-                    x0, x1 = 2, w
+                    x0, x1 = 6, w
                 else:
-                    x0, x1 = w, 2
+                    x0, x1 = w, 6
                 y = self.height() / 2
-                h = 0.75 * w
+                h = (w - 6) / 1.12
                 painter.setRenderHint(painter.Antialiasing)
-                painter.drawLine(x0, y - h, x1, y)
-                painter.drawLine(x1, y, x0, y + h)
+                painter.drawLines(
+                    QLine(x0, y - h, x1, y),
+                    QLine(x1, y, x0, y + h)
+                )
 
             def mouseReleaseEvent(self, event):
                 """Resize on left button"""
@@ -383,51 +475,41 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
         self.__splitter = self._Splitter(Qt.Horizontal, self)
         self.layout().addWidget(self.__splitter)
 
-    class _HiddenVerticalScrollArea(VerticalScrollArea):
-
-        def __init__(self, parent):
-            super().__init__(parent)
-            self.setFrameShape(QFrame.NoFrame)
-
-        def sizeHint(self):
-            if self.widget():
-                return self.widget().sizeHint()
-            else:
-                return super().sizeHint()
-
-        def minimumSizeHint(self):
-            if self.widget():
-                min_height = min(self.widget().minimumSizeHint().height(), 300)
-                return QSize(self.widget().minimumSizeHint().width(), min_height)
-            else:
-                return super().minimumSizeHint()
-
     def _insert_control_area(self):
-        if self.left_side_scrolling:
-            scroll_area = self._HiddenVerticalScrollArea(self)
-            self.__splitter.addWidget(scroll_area)
-            self.left_side = gui.vBox(scroll_area, spacing=0)
-            scroll_area.setWidget(self.left_side)
-        else:
-            self.left_side = gui.vBox(self.__splitter, spacing=0)
-        self.__splitter.setSizes([1])  # Smallest size allowed by policy
-        if self.buttons_area_orientation is not None:
-            self.controlArea = gui.vBox(self.left_side, addSpace=0)
-            self._insert_buttons_area()
-        else:
-            self.controlArea = self.left_side
+        self.left_side = gui.vBox(self.__splitter, addSpace=0)
         if self.want_main_area:
-            self.controlArea.setSizePolicy(
-                QSizePolicy.Fixed, QSizePolicy.MinimumExpanding)
-            m = 0
+            self.left_side.setSizePolicy(
+                QSizePolicy.MinimumExpanding, QSizePolicy.MinimumExpanding)
+
+            scroll_area = VerticalScrollArea(self.left_side)
+            scroll_area.setSizePolicy(QSizePolicy.MinimumExpanding,
+                                      QSizePolicy.Preferred)
+            self.controlArea = gui.vBox(scroll_area, spacing=0,
+                                        sizePolicy=(QSizePolicy.Fixed,
+                                                    QSizePolicy.MinimumExpanding))
+            scroll_area.setWidget(self.controlArea)
+
+            self.left_side.layout().addWidget(scroll_area)
+
+            m = 4, 4, 0, 4
         else:
-            m = 4
-        self.controlArea.layout().setContentsMargins(m, m, m, m)
+            self.controlArea = gui.vBox(self.left_side, spacing=0)
+
+            m = 4, 4, 4, 4
+
+        if self.buttons_area_orientation is not None:
+            self._insert_buttons_area()
+            self.buttonsArea.layout().setContentsMargins(4, 4, 0, 4)
+
+        self.controlArea.layout().setContentsMargins(*m)
 
     def _insert_buttons_area(self):
         self.buttonsArea = gui.widgetBox(
             self.left_side, addSpace=0, spacing=9,
-            orientation=self.buttons_area_orientation)
+            orientation=self.buttons_area_orientation,
+            sizePolicy=(QSizePolicy.MinimumExpanding,
+                        QSizePolicy.Maximum)
+        )
 
     def _insert_main_area(self):
         self.mainArea = gui.vBox(
@@ -437,8 +519,13 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
         self.__splitter.addWidget(self.mainArea)
         self.__splitter.setCollapsible(
             self.__splitter.indexOf(self.mainArea), False)
-        self.mainArea.layout().setContentsMargins(
-            0 if self.want_control_area else 4, 4, 4, 4)
+        if self.want_control_area:
+            self.mainArea.layout().setContentsMargins(
+                0, 4, 4, 4)
+            self.__splitter.setSizes([1, QWIDGETSIZE_MAX])
+        else:
+            self.mainArea.layout().setContentsMargins(
+                4, 4, 4, 4)
 
     def _create_default_buttons(self):
         # These buttons are inserted in buttons_area, if it exists
@@ -1113,6 +1200,7 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
 
     widgetStateChanged = Signal()
     blockingStateChanged = Signal(bool)
+    processingStateChanged = Signal(int)
     invalidatedStateChanged = Signal(bool)
     readyStateChanged = Signal(bool)
 
@@ -1396,36 +1484,6 @@ class _StatusBar(QStatusBar):
         painter.end()
 
 
-class Message:
-    """
-    A user message.
-
-    :param str text: Message text
-    :param str persistent_id:
-        A persistent message id.
-    :param icon: Message icon
-    :type icon: QIcon or QStyle.StandardPixmap
-    :param str moreurl:
-        An url to open when a user clicks a 'Learn more' button.
-
-    .. seealso:: :const:`OWBaseWidget.UserAdviceMessages`
-    """
-    #: QStyle.SP_MessageBox* pixmap enums repeated for easier access
-    Question = QStyle.SP_MessageBoxQuestion
-    Information = QStyle.SP_MessageBoxInformation
-    Warning = QStyle.SP_MessageBoxWarning
-    Critical = QStyle.SP_MessageBoxCritical
-
-    def __init__(self, text, persistent_id, icon=None, moreurl=None):
-        assert isinstance(text, str)
-        assert isinstance(icon, (type(None), QIcon, QStyle.StandardPixmap))
-        assert persistent_id is not None
-        self.text = text
-        self.icon = icon
-        self.moreurl = moreurl
-        self.persistent_id = persistent_id
-
-
 #: Input/Output flags (deprecated).
 #: --------------------------------
 #:
@@ -1631,7 +1689,8 @@ class StateInfo(QObject):
         elif isinstance(summary, StateInfo.Summary):
             assert_single_arg()
             if isinstance(summary, StateInfo.Empty):
-                summary = summary.updated(details="No data on input")
+                summary = summary.updated(details="No data on input",
+                                          brief='-')
             if summary.icon.isNull():
                 summary = summary.updated(icon=summary.default_icon("input"))
         elif isinstance(summary, str):
@@ -1688,7 +1747,8 @@ class StateInfo(QObject):
         elif isinstance(summary, StateInfo.Summary):
             assert_single_arg()
             if isinstance(summary, StateInfo.Empty):
-                summary = summary.updated(details="No data on output")
+                summary = summary.updated(details="No data on output",
+                                          brief='-')
             if summary.icon.isNull():
                 summary = summary.updated(icon=summary.default_icon("output"))
         elif isinstance(summary, str):
